from zope.interface import Interface

class IHorusUserClass(Interface):
    pass

class IHorusActivationClass(Interface):
    pass

class IHorusSession(Interface):
    pass

class IHorusLoginSchema(Interface):
    pass

class IHorusLoginForm(Interface):
    pass

class IHorusRegisterSchema(Interface):
    pass

class IHorusRegisterForm(Interface):
    pass

class IHorusForgotPasswordForm(Interface):
    pass

class IHorusForgotPasswordSchema(Interface):
    pass

class IHorusResetPasswordForm(Interface):
    pass

class IHorusResetPasswordSchema(Interface):
    pass

class IHorusProfileForm(Interface):
    pass

class IHorusProfileSchema(Interface):
    pass
